"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const aurelia_router_1 = require("aurelia-router");
const logger_1 = require("../development/logger");
const app_1 = require("./app");
const app_configuration_1 = require("./app-configuration");
const _VIEW_ANIMATION_READY_CLASS = "pitaya-view--animation-ready";
const _VIEW_ELEMENT_NAME = "pitaya-view";
const _VIEW_ACTIVE_LEAVE_CLASS = "au-leave-active";
const _ANIMATED_CALLBACK_NAME = "animated";
const _VIEW_STATUS_PRESET = {
    active: false,
    created: false,
    bound: false,
    attached: false,
    animated: false,
    detached: false,
    unbound: false
};
const _MDC_COLOR_VARIABLE_MAP = {
    "primary": "--mdc-theme-primary",
    "secondary": "--mdc-theme-secondary",
    "on-primary": "--mdc-theme-on-primary",
    "on-secondary": "--mdc-theme-on-secondary",
    "background": "--mdc-theme-background",
    "on-surface": "--mdc-theme-on-surface",
    "surface": "--mdc-theme-surface",
};
let _currentViewInstance = null;
class PitayaView {
    constructor() {
        this._animationTaskQueue = [];
        this._settings = {
            hierarchy: 0,
            theme: app_configuration_1.PitayaAppConfiguration.View.defaultTheme,
            redirect: {
                route: null,
                condition: null
            },
        };
        this._states = Object.assign({}, _VIEW_STATUS_PRESET);
        _wrapMethod.call(this, this._canActivate, "canActivate");
        _wrapMethod.call(this, this._activate, "activate");
        _wrapMethod.call(this, this._created, "created");
        _wrapMethod.call(this, this._bind, "bind");
        _wrapMethod.call(this, this._attached, "attached");
        _wrapMethod.call(this, this._detached, "detached");
        _wrapMethod.call(this, this._unbind, "unbind");
    }
    static get instance() {
        return _currentViewInstance;
    }
    get scrollingIsEnabled() {
        return this._viewModel.scrollingIsEnabled;
    }
    get rootElement() {
        return this._rootElement;
    }
    get states() {
        return this._states;
    }
    get innerElement() {
        return this._innerElement;
    }
    navigate(route, options) {
        return app_1.navigate(route, options);
    }
    navigateToRoute(route, params, options) {
        return app_1.navigateToRoute(route, params, options);
    }
    navigateBack() {
        app_1.navigateBack();
    }
    queueAnimationTask(task) {
        this._animationTaskQueue.push(task);
    }
    setTheme(color = this._settings.theme) {
        app_1.setStatusBarColor(_getColorFromMdcVariable(color));
    }
    enableScrolling(enable = true) {
        this._viewModel.scrollingIsEnabled = enable;
    }
    _canActivate(params, routeConfig, navigationInstruction) {
        const { redirect } = this._settings;
        if (!redirect.condition || !redirect.condition.call(this, this, redirect.route)) {
            return true;
        }
        return new aurelia_router_1.RedirectToRoute(redirect.route);
    }
    _activate(params, routeConfig, navigationInstruction) {
        this._settings.hierarchy = routeConfig.nav || 0;
        this._states.active = true;
    }
    _created(parentContext, context) {
        this._states.created = true;
        const model = context.controllers.find((controller) => {
            const { elementName } = controller.behavior;
            return elementName === _VIEW_ELEMENT_NAME;
        });
        this._viewModel = model.viewModel;
        _currentViewInstance = this;
    }
    _bind(bindingContext, overrideContext) {
        this._states.bound = true;
    }
    _attached() {
        this._states.attached = true;
        this._rootElement = this._viewModel.rootElement;
        this._innerElement = this._viewModel.innerElement;
        this._setAnimationListener();
    }
    _detached() {
        this._states.attached = false;
        this._states.detached = true;
    }
    _unbind() {
        this._states.unbound = false;
    }
    _willBeAnimated() {
        const { rootElement } = this;
        const siblings = [...rootElement.parentElement.children];
        return siblings.some((sibling) => sibling !== rootElement && sibling.classList.contains(_VIEW_ACTIVE_LEAVE_CLASS));
    }
    _setAnimationListener() {
        const { rootElement } = this;
        if (!this._willBeAnimated()) {
            this._onAnimationEnd();
            return;
        }
        rootElement.addEventListener("animationend", this._onAnimationEnd.bind(this), { once: true });
    }
    async _onAnimationEnd() {
        const queue = this._animationTaskQueue;
        while (queue.length) {
            await queue.pop()();
        }
        this.rootElement.classList.add(_VIEW_ANIMATION_READY_CLASS);
        this._states.animated = true;
        this[_ANIMATED_CALLBACK_NAME] && this[_ANIMATED_CALLBACK_NAME]();
    }
}
exports.PitayaView = PitayaView;
PitayaView.prototype["canActivate"] = function (...args) { };
PitayaView.prototype["activate"] = function (...args) { };
PitayaView.prototype["created"] = function (...args) { };
PitayaView.prototype["bind"] = function (...args) { };
PitayaView.prototype["attached"] = function (...args) { };
PitayaView.prototype["detached"] = function (...args) { };
PitayaView.prototype["unbind"] = function (...args) { };
exports.scrollTo = function (target, offset = 16) {
    const viewElement = _currentViewInstance.rootElement;
    const position = target instanceof HTMLElement ? Math.max(0, target.offsetTop - offset) : target;
    viewElement.scrollTo({
        behavior: "smooth",
        left: 0,
        top: position
    });
};
exports.enableScrolling = function (enable) {
    _currentViewInstance.enableScrolling(enable);
};
const _noop = () => { };
const _getColorFromMdcVariable = function (color) {
    if (color.startsWith("#") || color.startsWith("rgb")) {
        return color;
    }
    if (!(color in _MDC_COLOR_VARIABLE_MAP)) {
        logger_1.log.error("view.js _getColorFromMdcVariable()", `No MDC variable found that matches '${color}'`);
        return "";
    }
    return getComputedStyle(app_1.PitayaApp.instance.appView.rootElement).getPropertyValue(_MDC_COLOR_VARIABLE_MAP[color]);
};
const _wrapMethod = function (shadowMethod, methodName) {
    const originalMethod = this[methodName] || _noop;
    this.constructor.prototype[methodName] = (...args) => {
        shadowMethod.call(this, ...args);
        originalMethod.call(this, ...args);
    };
};
//# sourceMappingURL=view.js.map